#!/usr/bin/env python
import os
import sys
import struct

fname = 'themes/Silver/cursors/hand'
XCURSOR_MAGIC = 0x72756358L

class XCursorFile:
    def __init__(self, fname, verbose = 0):
        self.cursors = [];
        self.loaded_ok = 0;
        f = file(fname, 'rb');
        magic = struct.unpack("I", f.read(4))[0];
        if magic != XCURSOR_MAGIC:
            print "bad magic %x %x" % (magic, XCURSOR_MAGIC)
            return
        headerlen, version, ntocs = struct.unpack("III", f.read(3*4));
        if verbose:
            print "hlen %x ver %x ntocs %x" % (headerlen, version, ntocs)
        tocs = [];
        for i in range(ntocs):            
            type, subtype, position = struct.unpack("III", f.read(3*4));
            if verbose > 1:
                print " t %x subt %x pos %x" % (type, subtype, position)
            tocs.append( (type, subtype, position) );
        self.load_cursors(f, tocs, verbose)
        self.loaded_ok = 1 # yeah, this is bullshit, but no can do better ATM
        f.close();
    def load_cursors(self, f, tocs, verbose = 0):
        curs = [];
        for i in tocs:
            f.seek(i[2], 0);
            header, type, subtype, version = struct.unpack("IIII", f.read(4*4))
            if verbose:
                print "hdr %x t %x subt %x ver %x" % (header, type, subtype, version)
            if type == 0xfffd0002L: # XCURSOR_IMAGE_TYPE
                ac = XCursorFile.XCursorImage();
                ac.load_from_file(f, verbose);
                self.cursors.append(ac);
    def write_to(self, fname):
        f = file(fname, 'wb');
        data = struct.pack("4I", XCURSOR_MAGIC, 4*4, (1<<16) | 0, len(self.cursors));
        f.write(data);
        data_at = 4*4 + 3*4*len(self.cursors);
        for i in self.cursors:
            # Write tocs
            data = struct.pack("3I", 0xfffd0002L, i.get_subtype(), data_at)
            data_at += i.get_size();
            f.write(data)
        for i in self.cursors:
            # now write real data.
            data = struct.pack("4I", 4*4+5*4, 0xfffd0002L, i.get_subtype(), 1)
            f.write(data)
            i.save_to_file(f);
        
    def add_image(self, w, h, data):
        a = XCursorFile.XCursorImage()
        a.load_from_data(w, h, data)
        self.cursors.append(a)
        
    def get_sizes(self):
        sizes = []
        for i in self.cursors:
            size = '%dx%d' % (i.width, i.height)
            # print size
            if size not in sizes:
                sizes.append(size)
        return sizes
    def get_images_str(self, res):
        i_str = []; g = 0;
        for i in self.cursors:
            myres = "%dx%d" % (i.width, i.height)
            if myres == res:
                i_str.append("%2d (%d %d) %dms" % (g, i.xhot, i.yhot, i.delay))
                g+=1
        return i_str;
    def get_image_by_size(self, imgres, num):
        g = 0;
        for i in self.cursors:
            myres = "%dx%d" % (i.width, i.height)
            if myres == imgres:
                if g == num:
                    return i;
                g+=1;
        return None
                
    def rm_image_by_size(self, imgres, num):
        c_num = 0;
        for g in range(len(self.cursors)):
            i = self.cursors[g]
            myres = "%dx%d" % (i.width, i.height)
            if myres == imgres:
                if c_num == num:
                    del self.cursors[g]
                    return;
                c_num += 1
                
    class XCursorImage:
        def load_from_file(self, f, verbose):
            self.width, self.height, self.xhot, self.yhot, self.delay = struct.unpack("5I", f.read(5*4))
            if verbose > 10:
                print "%dx%d (%d %d) %d ms" % (self.width, self.height, self.xhot, self.yhot, self.delay)
            self.data = [];
	    loadstr = '%dI' % (self.width*self.height)
	    self.data = struct.unpack(loadstr, f.read(self.width*self.height*4))
            #loadstr = '%dI' % self.width;
            #for j in range(self.height):
            #    cur += struct.unpack(loadstr, f.read(self.width*4));
        def save_to_file(self, f):
            data = struct.pack("5I", self.width, self.height, self.xhot, self.yhot, self.delay)
            f.write(data)
            def packbytes(a): return struct.pack("I", a);
            data = "".join(map(packbytes, self.data));
            f.write(data)
            
        def load_from_data(self, w, h, data):
            self.width, self.height, self.xhot, self.yhot, self.delay = w, h, w/2, h/2, 50
            self.data = data
            
        def get_gdkp(self):
            rdata = "GdkP";
            rdata += struct.pack("!I", 24+self.width*self.height*4)
            rdata += struct.pack("!I", 0x1010002L)
            rdata += struct.pack("!I", self.width*4);
            rdata += struct.pack("!2I", self.width, self.height);
            def packbytes(a): return struct.pack("!I", ((a&0xFFFFFF)<<8) | (a>>24)); # ARGB -> RGBA conversion.
            #rdata += reduce(addstr, map(packbytes, self.data))
            rdata += "".join(map(packbytes, self.data))
            return rdata
        def get_subtype(self):
            return self.width
        def get_size(self):
            return 36 + self.width*self.height*4;
        
        

if __name__ == '__main__':
    import sys
    if len(sys.argv) > 1:
        fname = sys.argv[1]
    a = XCursorFile(fname, 11);

    a.write_to('lv2');
    #gddata= a.cursors[0].get_gdkp();
    #print len(gddata)
    #print a.get_sizes()
